@@ -1,5 +1,5 @@
 /* source: sysutils.c */
-/* Copyright Gerhard Rieger 2001-2008 */
+/* Copyright Gerhard Rieger 2001-2011 */
 /* Published under the GNU General Public License V.2, see file COPYING */
 
 /* translate socket addresses into human readable form */
@@ -16,6 +16,40 @@
 #include "utils.h"
 #include "sysutils.h"
 
+/* Substitute for Write():
+   Try to write all bytes before returning; this handles EINTR,
+   EAGAIN/EWOULDBLOCK, and partial write situations. The drawback is that this
+   function might block even with O_NONBLOCK option.
+   Returns <0 on unhandled error, errno valid
+   Will only return <0 or bytes
+*/
+ssize_t writefull(int fd, const void *buff, size_t bytes) {
+   size_t writt = 0;
+   ssize_t chk;
+   while (1) {
+      chk = Write(fd, (const char *)buff + writt, bytes - writt);
+      if (chk < 0) {
+	 switch (errno) {
+	 case EINTR:
+	 case EAGAIN:
+#if EAGAIN != EWOULDBLOCK
+	 case EWOULDBLOCK:
+#endif
+	    Warn4("write(%d, %p, "F_Zu"): %s", fd, (const char *)buff+writt, bytes-writt, strerror(errno));
+	    Sleep(1); continue;
+	 default: return -1;
+	 }
+      } else if (writt+chk < bytes) {
+	 Warn4("write(%d, %p, "F_Zu"): only wrote "F_Zu" bytes, trying to continue ",
+	       fd, (const char *)buff+writt, bytes-writt, chk);
+	 writt += chk;
+      } else {
+	 writt = bytes;
+	 break;
+      }
+   }
+   return writt;
+}
 
 #if WITH_UNIX
 void socket_un_init(struct sockaddr_un *sa) {
@@ -215,10 +249,14 @@ char *sockaddr_unix_info(const struct sockaddr_un *sa, socklen_t salen, char *bu
    } else
 #endif /* WITH_ABSTRACT_UNIXSOCKET */
    {
-      nextc =
-	 sanitize_string(sa->sun_path,
-			 MIN(UNIX_PATH_MAX, strlen(sa->sun_path)),
-			 ubuff, XIOSAN_DEFAULT_BACKSLASH_OCT_3);
+      if (salen <= XIOUNIXSOCKOVERHEAD) {
+	 nextc = sanitize_string ("<anon>", MIN(UNIX_PATH_MAX, strlen("<anon>")),
+				  ubuff, XIOSAN_DEFAULT_BACKSLASH_OCT_3);
+      } else {
+	 nextc = sanitize_string(sa->sun_path,
+				 MIN(UNIX_PATH_MAX, strlen(sa->sun_path)),
+				 ubuff, XIOSAN_DEFAULT_BACKSLASH_OCT_3);
+      }
       *nextc = '\0';
       strncpy(buff, ubuff, blen);
    }
@@ -374,6 +412,7 @@ char *sockaddr_inet6_info(const struct sockaddr_in6 *sa, char *buff, size_t blen
 }
 #endif /* WITH_IP6 */
 
+#if defined(HAVE_SETGRENT) && defined(HAVE_GETGRENT) && defined(HAVE_ENDGRENT)
 /* fill the list with the supplementary group ids of user.
    caller passes size of list in ngroups, function returns number of groups in
    ngroups.
@@ -400,6 +439,7 @@ int getusergroups(const char *user, gid_t *list, size_t *ngroups) {
    *ngroups = i;
    return 0;
 }
+#endif
 
 #if !HAVE_HSTRERROR
 const char *hstrerror(int err) {
