@@ -414,8 +414,10 @@ int _xioopen_foxec(int xioflags,	/* XIO_RDONLY etc. */
       gid_t group;
 
       if (withfork) {
-	 if (Signal(SIGCHLD, SIG_IGN) == SIG_ERR) {
-	    Warn1("signal(SIGCHLD, SIG_IGN): %s", strerror(errno));
+	 /* The child should have default handling for SIGCHLD. */
+	 /* In particular, it's not defined whether ignoring SIGCHLD is inheritable. */
+	 if (Signal(SIGCHLD, SIG_DFL) == SIG_ERR) {
+	    Warn1("signal(SIGCHLD, SIG_DFL): %s", strerror(errno));
 	 }
 
 #if HAVE_PTY
@@ -554,8 +556,8 @@ int _xioopen_foxec(int xioflags,	/* XIO_RDONLY etc. */
    } else
 #endif /* HAVE_PTY */
    if (usepipes) {
-      Close(rdpip[1]);
-      Close(wrpip[0]);
+      if (rw == XIO_RDONLY)  Close(rdpip[1]);
+      if (rw == XIO_WRONLY)  Close(wrpip[0]);
    } else {
       Close(sv[1]);
    }
